#    Copyright (C) 2012, Continuum Analytics
#    All rights reserved.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions are
#    met:
#
#        * Redistributions of source code must retain the above copyright
#           notice, this list of conditions and the following disclaimer.
#
#        * Redistributions in binary form must reproduce the above
#           copyright notice, this list of conditions and the following
#           disclaimer in the documentation and/or other materials provided
#           with the distribution.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import sys, os, ctypes
import traceback
from ctypes import wintypes
from os.path import isdir, join
import _winreg as reg
from system_path import (remove_from_system_path, add_to_system_path,
                         broadcast_environment_settings_change)

def broadcast_associations_change():
    """Broadcasts to the system indicating that file associations have changed"""
    SHCNE_ASSOCCHANGED = 0x8000000
    SHCNF_IDLIST = 0
    SHChangeNotify = ctypes.windll.shell32.SHChangeNotify
    SHChangeNotify.restype = None
    SHChangeNotify.argtypes = [wintypes.LONG, wintypes.UINT, wintypes.LPCVOID,
                               wintypes.LPCVOID]

    SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, None, None)


def set_file_associations():
    """Sets .py and .pyw file associations for the current Python"""
    key = reg.CreateKey(reg.HKEY_CLASSES_ROOT, '.py')
    try:
        reg.SetValueEx(key, None, 0, reg.REG_SZ, 'Python.File')
        reg.SetValueEx(key, 'Content Type', 0, reg.REG_SZ, 'text/plain')
    finally:
        reg.CloseKey(key)

    key = reg.CreateKey(reg.HKEY_CLASSES_ROOT, '.pyw')
    try:
        reg.SetValueEx(key, None, 0, reg.REG_SZ, 'Python.NoConFile')
        reg.SetValueEx(key, 'Content Type', 0, reg.REG_SZ, 'text/plain')
    finally:
        reg.CloseKey(key)

    key = reg.CreateKey(reg.HKEY_CLASSES_ROOT, 'Python.File')
    try:
        key2 = reg.CreateKey(key, 'DefaultIcon')
        try:
            reg.SetValueEx(key2, None, 0, reg.REG_SZ,
                           join(sys.prefix, 'DLLs', 'py.ico'))
        finally:
            reg.CloseKey(key2)
        key2 = reg.CreateKey(key, 'shell\\open\\command')
        try:
            reg.SetValueEx(key2, None, 0, reg.REG_SZ,
                           '"%s" "%%1" %%*' % join(sys.prefix, 'python.exe'))
        finally:
            reg.CloseKey(key2)
    finally:
        reg.CloseKey(key)

    key = reg.CreateKey(reg.HKEY_CLASSES_ROOT, 'Python.NoConFile')
    try:
        key2 = reg.CreateKey(key, 'DefaultIcon')
        try:
            reg.SetValueEx(key2, None, 0, reg.REG_SZ,
                           join(sys.prefix, 'DLLs', 'py.ico'))
        finally:
            reg.CloseKey(key2)
        key2 = reg.CreateKey(key, 'shell\\open\\command')
        try:
            reg.SetValueEx(key2, None, 0, reg.REG_SZ,
                           '"%s" "%%1" %%*' % join(sys.prefix, 'pythonw.exe'))
        finally:
            reg.CloseKey(key2)
    finally:
        reg.CloseKey(key)

    broadcast_associations_change()

def set_python_core_registry():
    key = reg.CreateKey(reg.HKEY_LOCAL_MACHINE,
            r'SOFTWARE\Python\PythonCore\%s.%s' % (sys.version_info[0], sys.version_info[1]))
    try:
        key2 = reg.CreateKey(key, r'Help\Main Python Documentation')
        try:
            helpfile = join(sys.prefix, 'Doc', 'python%s%s%s.chm' % (sys.version_info[0], sys.version_info[1], sys.version_info[2]))
            reg.SetValueEx(key2, None, 0, reg.REG_SZ, helpfile)
        finally:
            reg.CloseKey(key2)
        key2 = reg.CreateKey(key, 'InstallPath')
        try:
            reg.SetValueEx(key2, None, 0, reg.REG_SZ, sys.prefix + '\\')
        finally:
            reg.CloseKey(key2)
        key2 = reg.CreateKey(key, 'InstallPath\\InstallGroup')
        try:
            reg.SetValueEx(key2, None, 0, reg.REG_SZ,
                   'Python %s.%s' % (sys.version_info[0], sys.version_info[1]))
        finally:
            reg.CloseKey(key2)
        key2 = reg.CreateKey(key, 'PythonPath')
        try:
            pythonpath = join(sys.prefix, 'Lib')
            pythonpath += os.pathsep + join(sys.prefix, 'DLLs')
            if isdir(os.pathsep + join(sys.prefix, 'Lib', 'lib-tk')):
                pythonpath += os.pathsep + join(sys.prefix, 'Lib', 'lib-tk')
            reg.SetValueEx(key2, None, 0, reg.REG_SZ, pythonpath)
        finally:
            reg.CloseKey(key2)

    finally:
        reg.CloseKey(key)

def ask_user_permission():
    """Asks the user if it is ok to set the current default Python"""
    MB_ICONQUESTION = 0x00000020L
    MB_YESNO = 0x00000004L
    IDYES = 6
    IDNO = 7
    title = u'Set Default System Python'
    msg = u'Would you like to change the default system Python to the following?'
    msg += u'\n\n%s' % sys.prefix
    ret = ctypes.windll.user32.MessageBoxW(0, msg, title, MB_YESNO|MB_ICONQUESTION)
    return ret == IDYES

def error_dialog(msg, title):
    MB_ICONERROR = 0x00000010L
    MB_OK = 0
    ctypes.windll.user32.MessageBoxW(0, unicode(msg), unicode(title),
                                     MB_OK|MB_ICONERROR)

def info_dialog(msg, title):
    MB_ICONINFORMATION = 0x00000040L
    MB_OK = 0
    ctypes.windll.user32.MessageBoxW(0, unicode(msg), unicode(title),
                                     MB_OK|MB_ICONINFORMATION)

def set_current_python_as_system_default():
    """Changes the default system Python, including PATH, file associateions, etc."""
    # 1) Remove any existing Python and Python\Scripts from %PATH%, and add
    #    Python and Python\Scripts.
    #    HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\Session Manager\Environment
    #    HKEY_CURRENT_USER\Environment
    # 2) Associate .py files with python.exe, .pyw files with pythonw.exe
    # 3) Set the registry entries in HKEY_LOCAL_MACHINE\SOFTWARE\Python\PythonCore\2.7

    # 1) Change the system path
#    remove_from_system_path('python.exe')
#    remove_from_system_path('../python.exe', basename='Scripts')
    add_to_system_path([sys.prefix, join(sys.prefix, 'Scripts')])

    broadcast_environment_settings_change()
    # 2) Change the file associations
    set_file_associations()
    # 3) Change the registry entries for PythonCore
    set_python_core_registry()

if __name__ == '__main__':
    if sys.argv.count('--no-uac') > 0:
        # If UAC escalation is disabled, go directly to setting the default
        # Python
        try:
            if sys.argv.count('--no-user-question') == 0:
                if not ask_user_permission():
                    sys.exit(1)

            set_current_python_as_system_default()
            if not sys.argv.count('--no-dialog') > 0:
                info_dialog('Default Python has been set to:\n\n%s' %
                            sys.prefix, 'Default Python')
        except:
            error_dialog(traceback.format_exc(), 'Error Setting Default Python')
            sys.exit(1)
    else:
        # First ask permission if needed, then proceed with UAC escalation
        if sys.argv.count('--no-user-question') == 0:
            if not ask_user_permission():
                sys.exit(1)

        from uac import uac_elevate_privileges
        sys.exit(uac_elevate_privileges(
                __file__,
                '--no-uac --no-user-question' +
                (' --no-dialog' if sys.argv.count('--no-dialog') else '')))
