__all__ = ['remove_from_system_path', 'add_to_system_path',
           'broadcast_environment_settings_change']

import _winreg as reg
import os, ctypes
from os import path
from ctypes import wintypes

HWND_BROADCAST = 0xffff
WM_SETTINGCHANGE = 0x001A
SMTO_ABORTIFHUNG = 0x0002
SendMessageTimeout = ctypes.windll.user32.SendMessageTimeoutW
SendMessageTimeout.restype = None #wintypes.LRESULT
SendMessageTimeout.argtypes = [wintypes.HWND, wintypes.UINT, wintypes.WPARAM,
                               wintypes.LPCWSTR, wintypes.UINT, wintypes.UINT,
                               ctypes.POINTER(wintypes.DWORD)]

def sz_expand(value, value_type):
    if value_type == reg.REG_EXPAND_SZ:
        return reg.ExpandEnvironmentStrings(value)
    else:
        return value

def remove_from_system_path(relative_file, basename=None, path_env_var='PATH'):
    """Removes all entries from the path with the specified file existing
       relative to the path, and whose basename is as provided.

       You must call broadcast_environment_settings_change() after you are finished
       manipulating the environment with this and other functions.

       For example,
         # Remove main Python dir from PATH
         remove_from_system_path('python.exe')
         # Remove Python scripts dir from PATH
         remove_from_system_path('../python.exe', 'Scripts')
    """
    envkeys = [(reg.HKEY_LOCAL_MACHINE,
            r'SYSTEM\CurrentControlSet\Control\Session Manager\Environment'),
            (reg.HKEY_CURRENT_USER, r'Environment')]
    for root, keyname in envkeys:
        key = reg.OpenKey(root, keyname, 0,
                reg.KEY_QUERY_VALUE|reg.KEY_SET_VALUE)
        try:
            reg_value = reg.QueryValueEx(key, path_env_var)

            any_change = False
            results = []
            for v in reg_value[0].split(os.pathsep):
                vexp = sz_expand(v, reg_value[1])
                skip = False
                # Check if the required file exists (e.g. 'python.exe'
                # or '../python.exe')
                if path.isfile(path.join(vexp, relative_file)):
                    if basename is not None:
                        # Check if the basename of the path matches
                        # the requirement (e.g. 'bin' or 'Scripts')
                        if (path.basename(path.abspath(vexp)).lower() ==
                                         basename.lower()):
                            skip = True
                    else:
                        skip = True
                # Append the original unexpanded version to the results
                if skip:
                    any_change = True
                else:
                    results.append(v)

            modified_path = os.pathsep.join(results)
            if any_change:
                reg.SetValueEx(key, path_env_var, 0, reg_value[1], modified_path)
        finally:
            reg.CloseKey(key)

def add_to_system_path(paths, allusers=True, path_env_var='PATH'):
    """Adds the requested paths to the system PATH variable.

       You must call broadcast_environment_settings_change() after you are
       finished manipulating the environment with this and other functions.
    """
    if allusers:
        # All Users
        root, keyname = (reg.HKEY_LOCAL_MACHINE,
            r'SYSTEM\CurrentControlSet\Control\Session Manager\Environment')
    else:
        # Just Me
        root, keyname = (reg.HKEY_CURRENT_USER, r'Environment')
    key = reg.OpenKey(root, keyname, 0,
            reg.KEY_QUERY_VALUE|reg.KEY_SET_VALUE)
    try:
        reg_value = reg.QueryValueEx(key, path_env_var)
        new_paths = None
        for p in paths:
            p = path.abspath(p)
            if not path.isdir(p):
                raise RuntimeError('Directory "%s" does not exist, cannot add '
                                   'it to the path' % p)
            if new_paths:
                new_paths = new_paths + os.pathsep + p
            else:
                new_paths = p
        reg.SetValueEx(key, path_env_var, 0, reg_value[1], new_paths + os.pathsep + reg_value[0])
    finally:
        reg.CloseKey(key)

def broadcast_environment_settings_change():
    """ Broadcasts to the system indicating that master environment variables
    have changed.

    This must be called after using the other functions in this module to
    manipulate environment variables.
    """
    SendMessageTimeout(HWND_BROADCAST, WM_SETTINGCHANGE, 0, u'Environment',
                SMTO_ABORTIFHUNG, 5000, ctypes.pointer(wintypes.DWORD()))
